package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.generated.messages.Messages;
import ca.canadapost.cpcdp.shipping.generated.shipment.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetShipmentPublicKeyInfo Canada Post service.
 * <p> 
 * The GetShipmentPublicKeyInfo service is used to retrieve the public key info in XML 
 * format for a shipment created by a prior "create shipment" call.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetShipmentDetails Request.  <br>
 *  USAGE:<pre>
 *        GetShipmentPublicKeyInfo client = new GetShipmentPublicKeyInfo(username, password);
 *        ClientResponse response = client.getShipmentPublicKeyInfo();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetShipmentPublicKeyInfo {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
		
    	// Use link from CreateShipment
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/shipment/340531309186521749/qrCode";
    	
		// Execute GetShipmentPublicKeyInfo Request
        GetShipmentPublicKeyInfo myClient = new GetShipmentPublicKeyInfo(username, password);
        ClientResponse resp = myClient.getShipmentPublicKeyInfo();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(ShipmentPublicKeyInfo.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches ShipmentPublicKeyInfo schema.
            if (entity instanceof ShipmentPublicKeyInfo) {
            	ShipmentPublicKeyInfo shipmentPublicKeyInfo = (ShipmentPublicKeyInfo) entity;
            	System.out.println("Expiry Date: " + shipmentPublicKeyInfo.getExpiryDate());
            	System.out.println("Url: " + shipmentPublicKeyInfo.getUrl());
            	if(shipmentPublicKeyInfo.getQrCode() != null) {
            		//write QR code to file
            		Path pathToQRCodeImage = Paths.get("qr-code.jpeg");
            		Files.write(pathToQRCodeImage, shipmentPublicKeyInfo.getQrCode());
            		System.out.println("QR code image created at: " + pathToQRCodeImage.toAbsolutePath());
            	}                
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetShipmentPublicKeyInfo(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getShipmentPublicKeyInfo() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.shipment-v8+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
