package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.generated.messages.Messages;
import ca.canadapost.cpcdp.shipping.generated.shipment.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetShipmentReceipt Canada Post service.
 * <p> 
 * This call is used to get settlement information in XML format for a No Manifest Required shipment that was paid by credit card or 
 * third party account.  The data in the response can be formatted to print a credit card receipt.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetShipmentReceipt Request.  <br>
 *  USAGE:<pre>
 *        GetShipmentReceiptClient client = new GetShipmentReceiptClient(username, password);
 *        ClientResponse response = client.getShipmentReceipt();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetShipmentReceiptClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
		
    	// Use link from CreateShipment
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/shipment/340531309186521749/receipt";
		
		// Execute GetShipmentDetails Request
        GetShipmentReceiptClient myClient = new GetShipmentReceiptClient(username, password);
        ClientResponse resp = myClient.getShipmentDetails();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(ShipmentReceipt.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches ShipmentDetails schema.
            if (entity instanceof ShipmentReceipt) {
            	ShipmentReceipt shipmentReceipt = (ShipmentReceipt) entity;
            	
            	if(shipmentReceipt.getCcReceiptDetails() != null){
            		ShipmentReceipt.CcReceiptDetails receiptDetails = shipmentReceipt.getCcReceiptDetails();
                	System.out.println("Merchant Name: " + receiptDetails.getMerchantName());                
                	System.out.println("Merchant URL: " + receiptDetails.getMerchantUrl());                
                	System.out.println("Name on Card: " + receiptDetails.getNameOnCard());
                	System.out.println("Auth Code: " + receiptDetails.getAuthCode());
                	System.out.println("Auth Timestamp: " + receiptDetails.getAuthTimestamp());
                	System.out.println("Card Type: " + receiptDetails.getCardType());
                	System.out.println("Chareg Amount: " + receiptDetails.getChargeAmount());
                	System.out.println("Currency: " + receiptDetails.getCurrency());
                	System.out.println("Transaction Type: " + receiptDetails.getTransactionType());
            	}
            	
            	if(shipmentReceipt.getSupplierAccountReceiptDetails() != null){
            		ShipmentReceipt.SupplierAccountReceiptDetails receiptDetails = shipmentReceipt.getSupplierAccountReceiptDetails();
                	System.out.println("Merchant Name: " + receiptDetails.getMerchantName());                
                	System.out.println("Merchant URL: " + receiptDetails.getMerchantUrl());                
                	System.out.println("Auth Code: " + receiptDetails.getAuthCode());
                	System.out.println("Auth Timestamp: " + receiptDetails.getAuthTimestamp());
                	System.out.println("Supplier Id: " + receiptDetails.getSupplierId());
                	System.out.println("Chareg Amount: " + receiptDetails.getChargeAmount());
                	System.out.println("Currency: " + receiptDetails.getCurrency());
                	System.out.println("Transaction Type: " + receiptDetails.getTransactionType());
            	}
            	
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetShipmentReceiptClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getShipmentDetails() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.shipment-v8+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
