package ca.canadapost.cpcdp.shipping;


import ca.canadapost.cpcdp.shipping.generated.shipment.ReferencesType;
import ca.canadapost.cpcdp.shipping.generated.messages.Messages;
import ca.canadapost.cpcdp.shipping.generated.shipment.*;
import ca.canadapost.cpcdp.shipping.generated.shipment.ParcelCharacteristicsType.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;


import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.GregorianCalendar; 
import java.util.Properties;

import java.math.*;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

/**
 * Sample code for the ShipmentRefundRequest Canada Post service.
 * <p>
 * The ShipmentRefundRequest service is used to request a refund for 
 * a shipment that has been transmitted. Making this call indicates 
 * that the previously printed label is spoiled or will otherwise not be used.
 * 
 * Note that these are requests for refund, as refunding a shipment requires 
 * proper verification before being actioned (i.e. ensure label has not been used, for example).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for RequestShipmentRefund Request.  <br>
 *  USAGE:<pre>
 *        RequestShipmentRefundClient client = new RequestShipmentRefund(username, password);
 *        ClientResponse response = client.createShipmentRefundRequest(request);
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class RequestShipmentRefundClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
    	
    	// REST URL
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/shipment/340531309186521749/refund";
    	
		// Create XML Request Object
        ShipmentRefundRequest shipmentRefundRequest = new ShipmentRefundRequest();
        shipmentRefundRequest.setEmail("user@host.com");
		
		// ExecuteRequest
        RequestShipmentRefundClient myClient = new RequestShipmentRefundClient(username, password);
        ClientResponse resp = myClient.createShipmentRefundRequest(shipmentRefundRequest);
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(ShipmentRefundRequestInfo.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches CreateShipmentInfo schema.
            if (entity instanceof ShipmentRefundRequestInfo) {
            	ShipmentRefundRequestInfo shipmentRefundRequestInfo = (ShipmentRefundRequestInfo) entity;
            	System.out.println("Service Ticket ID: " + shipmentRefundRequestInfo.getServiceTicketId());                
            	System.out.println("Service Ticket Date: " + shipmentRefundRequestInfo.getServiceTicketDate());
                
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public RequestShipmentRefundClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse createShipmentRefundRequest(Object xml) throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource
        		.accept("application/vnd.cpc.shipment-v8+xml")
        		.header("Content-Type", "application/vnd.cpc.shipment-v8+xml")
        		.header("Source-App", "EXT")
        		.header("Consumer-Id", "1")
        		.acceptLanguage("en-CA")
        		.post(ClientResponse.class, xml);
    }

    public void close() {
        aClient.destroy();
    }
}
